@@ -1,23 +1,28 @@
 import requests
 import json
-from typing import List, Dict, Union
+from typing import List, Dict, Union, Type
 from .manga import Manga, MangaTag
 from .chapter import Chapter
 from .group import Group
 from .user import User
 from .author import Author
+from .cover import Cover
 from .network import NetworkChapter
 from .search import SearchMapping
 
 
-class APIError(Exception):
+class MDException(Exception):
+    pass
+
+
+class APIError(MDException):
     def __init__(self, r):
         self.status = r.status_code
         self.data = None
         try:
             self.data = r.json()
-        except Exception as e:
-            print(e)
+        except json.decoder.JSONDecodeError:
+            self.data = r.content
 
 
 class NoContentError(APIError):
@@ -28,11 +33,11 @@ class LoginError(APIError):
     pass
 
 
-class NotLoggedInError(Exception):
+class NotLoggedInError(MDException):
     pass
 
 
-class NoResultsError(Exception):
+class NoResultsError(MDException):
     pass
 
 
@@ -53,11 +58,17 @@ class MangaDex:
         post = self.session.post(f"{self.api}/auth/login", data=json.dumps(credentials))
         return self._store_token(post)
 
+    def login_token(self, token: str) -> bool:
+        """Logs in to MangaDex using a refresh token."""
+        credentials = {"token": token}
+        post = self.session.post(f"{self.api}/auth/refresh", data=json.dumps(credentials))
+        return self._store_token(post)
+
     def logout(self):
         """Resets the current session."""
         self.__init__()
 
-    def refresh(self, token=None) -> bool:
+    def refresh_session(self, token: str = None) -> bool:
         """Refreshes the session using the refresh token."""
         if not self.login_success:
             raise NotLoggedInError
@@ -66,7 +77,7 @@ class MangaDex:
         post = self.session.post(f"{self.api}/auth/refresh", data=json.dumps(data))
         return self._store_token(post)
 
-    def check(self) -> bool:
+    def check_session(self) -> bool:
         """Checks if the stored Authorization token is still valid."""
         req = self.session.get(f"{self.api}/auth/check")
         if req.status_code == 200:
@@ -88,29 +99,29 @@ class MangaDex:
             self.session.headers["Authorization"] = resp["token"]["session"]
             return True
 
-    def get_manga(self, id_: str) -> Manga:
+    def get_manga(self, uuid: str) -> Manga:
         """Gets a manga with a specific uuid."""
-        req = self.session.get(f"{self.api}/manga/{id_}")
+        req = self.session.get(f"{self.api}/manga/{uuid}")
         if req.status_code == 200:
             resp = req.json()
             return Manga(resp["data"], resp["relationships"], self)
-        elif req.status_code == 204:
+        elif req.status_code == 404:
             raise NoContentError(req)
         else:
             raise APIError(req)
 
-    def get_chapter(self, id_: str) -> Chapter:
+    def get_chapter(self, uuid: str) -> Chapter:
         """Gets a chapter with a specific uuid."""
-        req = self.session.get(f"{self.api}/chapter/{id_}")
+        req = self.session.get(f"{self.api}/chapter/{uuid}")
         if req.status_code == 200:
             resp = req.json()
             return Chapter(resp["data"], resp["relationships"], self)
-        elif req.status_code == 204:
+        elif req.status_code == 404:
             raise NoContentError(req)
         else:
             raise APIError(req)
 
-    def get_chapters(self, ids: list) -> List[Chapter]:
+    def get_chapters(self, ids: List[str]) -> List[Chapter]:
         """Gets chapters with specific uuids."""
         chapters = []
         sub = [ids[x:x+100] for x in range(0, len(ids), 100)]
@@ -137,6 +148,23 @@ class MangaDex:
             params.pop("offset")
         return self._retrieve_pages(f"{self.api}/manga/{mg.id}/feed", Chapter, call_limit=100, params=params)
 
+    def get_manga_covers(self, mg: Manga, params: dict = None) -> List[Cover]:
+        """Gets covers associated with a specific Manga."""
+        params = params or {}
+        params["manga[]"] = mg.id
+        return self._retrieve_pages(f"{self.api}/cover", Cover, call_limit=100, params=params)
+
+    def get_cover(self, uuid: str) -> Cover:
+        """Gets a cover with a specific uuid."""
+        req = self.session.get(f"{self.api}/cover/{uuid}")
+        if req.status_code == 200:
+            resp = req.json()
+            return Cover(resp["data"], resp["relationships"], self)
+        elif req.status_code == 404:
+            raise NoContentError(req)
+        else:
+            raise APIError(req)
+
     def read_chapter(self, ch: Chapter, force_443: bool = False) -> NetworkChapter:
         """Pulls a chapter from the MD@H Network."""
         data = {"forcePort443": force_443}
@@ -156,26 +184,26 @@ class MangaDex:
         else:
             raise APIError(req)
 
-    def get_group(self, id_: str) -> Group:
+    def get_group(self, uuid: str) -> Group:
         """Gets a group with a specific uuid."""
-        req = self.session.get(f"{self.api}/group/{id_}")
+        req = self.session.get(f"{self.api}/group/{uuid}")
         if req.status_code == 200:
             resp = req.json()
             return Group(resp["data"], None, self)
-        elif req.status_code == 204:
+        elif req.status_code == 404:
             raise NoContentError(req)
         else:
             raise APIError(req)
 
-    def get_user(self, id_: str = "me") -> User:
+    def get_user(self, uuid: str = "me") -> User:
         """Gets an user with a specific uuid."""
-        if id_ == "me" and not self.login_success:
+        if uuid == "me" and not self.login_success:
             raise NotLoggedInError
-        req = self.session.get(f"{self.api}/user/{id_}")
+        req = self.session.get(f"{self.api}/user/{uuid}")
         if req.status_code == 200:
             resp = req.json()
             return User(resp["data"], self)
-        elif req.status_code == 204:
+        elif req.status_code == 404:
             raise NoContentError(req)
         else:
             raise APIError(req)
@@ -191,51 +219,48 @@ class MangaDex:
         if not self.login_success:
             raise NotLoggedInError
         params = params or {}
-        if "limit" in params:
-            params.pop("limit")
-        if "offset" in params:
-            params.pop("offset")
         return self._retrieve_pages(f"{self.api}/user/follows/manga/feed", Chapter, call_limit=100,
                                     limit=limit, params=params)
 
-    def get_author(self, id_: str) -> Author:
+    def get_author(self, uuid: str) -> Author:
         """Gets an author with a specific uuid"""
-        req = self.session.get(f"{self.api}/author/{id_}")
+        req = self.session.get(f"{self.api}/author/{uuid}")
         if req.status_code == 200:
             resp = req.json()
             return Author(resp["data"], resp["relationships"], self)
-        elif req.status_code == 204:
+        elif req.status_code == 404:
             raise NoContentError(req)
         else:
             raise APIError(req)
 
-    def transform_ids(self, obj: str, content: list) -> Dict:
+    def transform_ids(self, obj: str, content: List[int]) -> Dict:
         """Gets uuids from legacy ids."""
         data = {"type": obj, "ids": content}
         post = self.session.post(f"{self.api}/legacy/mapping", data=json.dumps(data))
         if post.status_code == 200:
             resp = post.json()
             return {x["data"]["attributes"]["legacyId"]: x["data"]["attributes"]["newId"] for x in resp}
-        elif post.status_code == 204:
-            raise NoContentError(post)
         else:
             raise APIError(post)
 
-    def search(self, obj: str, params: dict, limit: int = 100) -> List[Union[Manga, Chapter, Group, Author]]:
+    def search(self, obj: str, params: dict,
+               limit: int = 100) -> List[Union[Manga, Chapter, Group, Author, Cover]]:
         """Searches an object."""
-        if "limit" in params:
-            params.pop("limit")
-        if "offset" in params:
-            params.pop("offset")
         m = SearchMapping(obj)
         return self._retrieve_pages(f"{self.api}{m.path}", m.object, limit=limit, call_limit=100, params=params)
 
-    def _retrieve_pages(self, url: str, obj, limit: int = 0, call_limit: int = 500, params: dict = None):
+    def _retrieve_pages(self, url: str, obj: Type[Union[Manga, Chapter, Group, Author, Cover]],
+                        limit: int = 0, call_limit: int = 500,
+                        params: dict = None) -> List[Union[Manga, Chapter, Group, Author, Cover]]:
         params = params or {}
         data = []
         offset = 0
         resp = None
         remaining = True
+        if "limit" in params:
+            params.pop("limit")
+        if "offset" in params:
+            params.pop("offset")
         while remaining:
             p = {"limit": limit if limit <= call_limit and limit != 0 else call_limit, "offset": offset}
             p = {**p, **params}
