@@ -2,8 +2,9 @@ from .manga import Manga
 from .chapter import Chapter
 from .group import Group
 from .author import Author
-classes = {"manga": Manga, "chapter": Chapter, "group": Group, "author": Author}
-paths = {"manga": "/manga", "chapter": "/chapter", "group": "/group", "author": "/author"}
+from .cover import Cover
+classes = {"manga": Manga, "chapter": Chapter, "group": Group, "author": Author, "cover": Cover}
+paths = {"manga": "/manga", "chapter": "/chapter", "group": "/group", "author": "/author", "cover": "/cover"}
 
 
 class SearchMapping:
